///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

// The portions of this code that relate to the text editing are basically
// taken from the built in wxWidgets text editor for cells: wxGridCellTextEditor.

#include "CEImagesetConfig.h"

// CEGUI includes
#include <CEGUI.h>

using namespace CEGUI;

// Declare the instance
CEImagesetConfig* CEImagesetConfig::m_instance = NULL;
wxString CEImagesetConfig::m_configName = wxT("");
wxString CEImagesetConfig::m_configDir = wxT("");


//-----------------------------------------------------------------------

void CEImagesetConfig::setConfigName(wxString configName)
{
		m_configName = configName;
}

wxString CEImagesetConfig::getConfigName()
{
		return m_configName;
}

void CEImagesetConfig::setConfigDir(wxString configDir)
{
		m_configDir = configDir;
}

//-----------------------------------------------------------------------
CEImagesetConfig* CEImagesetConfig::GetInstancePtr()
{
    // Create instance, if not done yet
    if (!m_instance)
    {
		if (m_configName.IsEmpty())
			m_configName = wxT(DEFAULT_CONFIG_FILE_NAME);

		#ifdef WIN32
			wxString ConfigGlobalFileName = wxT("");
			wxString ConfigLocalFileName = wxGetCwd()+wxFileName::GetPathSeparator()+m_configName+wxT(".ini");
		#else
			wxString ConfigLocalFileName = wxT(".")+m_configName;
			wxString ConfigGlobalFileName = m_configName;
		#endif

		m_instance = new CEImagesetConfig(m_configName,ConfigLocalFileName,ConfigGlobalFileName);
    }
    return m_instance;
}
	

//-----------------------------------------------------------------------


CEImagesetConfig::CEImagesetConfig(const wxString& appName, const wxString& ConfigLocalFileName, const wxString& ConfigGlobalFileName) :
	wxFileConfig(appName, _T(""), ConfigLocalFileName, ConfigGlobalFileName, wxCONFIG_USE_LOCAL_FILE)
{

}

//-----------------------------------------------------------------------
CEImagesetConfig::~CEImagesetConfig()
{
	delete wxFileConfig::Set((wxFileConfig*)NULL);
}

//-----------------------------------------------------------------------
void CEImagesetConfig::createDefaultEditorConfig()
{
	Write(wxT(CONFIG_DONT_RUN_INSTALL_WIZARD), wxT(""));

	long defaultColor = 127;
	Write(wxT(CONFIG_COLOR_RED), defaultColor);
	Write(wxT(CONFIG_COLOR_GREEN), defaultColor);
	Write(wxT(CONFIG_COLOR_BLUE), defaultColor);
}

//-----------------------------------------------------------------------
void CEImagesetConfig::wizardSaveResourcePaths()
{
	wxString path;
	if( m_configDir.IsEmpty())
		path = wxGetCwd()+wxFileName::GetPathSeparator()+wxT("samples")+wxFileName::GetPathSeparator();
	else
		path = m_configDir+wxFileName::GetPathSeparator();

	CEImagesetConfig::GetInstancePtr()->SetPath(wxT(CONFIG_GROUP_ENTRY));
	Write(wxT(CONFIG_DEFAULT_GROUP_NAME_IMAGESET),path+wxT(CONFIG_DEFAULT_GROUP_NAME_IMAGESET)+wxFileName::GetPathSeparator());
	Write(wxT(CONFIG_DEFAULT_GROUP_NAME_FONT),path+wxT(CONFIG_DEFAULT_GROUP_NAME_FONT)+wxFileName::GetPathSeparator());
	Write(wxT(CONFIG_DEFAULT_GROUP_NAME_SCHEME),path+wxT(CONFIG_DEFAULT_GROUP_NAME_SCHEME)+wxFileName::GetPathSeparator());
	Write(wxT(CONFIG_DEFAULT_GROUP_NAME_LOOKNFEELS),path+wxT(CONFIG_DEFAULT_GROUP_NAME_LOOKNFEELS)+wxFileName::GetPathSeparator());
	Write(wxT(CONFIG_DEFAULT_GROUP_NAME_LAYOUT),path+wxT(CONFIG_DEFAULT_GROUP_NAME_LAYOUT)+wxFileName::GetPathSeparator());
	Write(wxT(CONFIG_DEFAULT_GROUP_NAME_SCRIPT),path+wxT(CONFIG_DEFAULT_GROUP_NAME_SCRIPT)+wxFileName::GetPathSeparator());
	CEImagesetConfig::GetInstancePtr()->SetPath(wxT("/"));

	Write(wxT(CONFIG_DEFAULT_GROUP_IMAGESET),wxT(CONFIG_DEFAULT_GROUP_NAME_IMAGESET));
	Write(wxT(CONFIG_DEFAULT_GROUP_FONT),wxT(CONFIG_DEFAULT_GROUP_NAME_FONT));
	Write(wxT(CONFIG_DEFAULT_GROUP_SCHEME),wxT(CONFIG_DEFAULT_GROUP_NAME_SCHEME));
	Write(wxT(CONFIG_DEFAULT_GROUP_LOOKNFEELS),wxT(CONFIG_DEFAULT_GROUP_NAME_LOOKNFEELS));
	Write(wxT(CONFIG_DEFAULT_GROUP_LAYOUT),wxT(CONFIG_DEFAULT_GROUP_NAME_LAYOUT));
	Write(wxT(CONFIG_DEFAULT_GROUP_SCRIPT),wxT(CONFIG_DEFAULT_GROUP_NAME_SCRIPT));

}
