///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////
#ifdef HAVE_CONFIG_H
#   include "config.h"
#endif

#include <CEGUI.h>

#include "LayoutGLCanvas.h"
#include "EditorGLCanvas.h"
#include "PropertiesPanel.h"
#include "EditorView.h"
#include "CEGUIHelper.h"
#include "ElasticBox.h"
#include "ElasticBoxWindowRenderer.h"
#include "CEImagesetConfig.h"

#define S_(X) #X
#define STRINGIZE(X) S_(X)

using namespace CEGUI;

//-----------------------------------------------------------------------
BEGIN_EVENT_TABLE( LayoutGLCanvas, wxGLCanvas )
    EVT_SIZE ( LayoutGLCanvas::OnResize )
    EVT_PAINT   ( LayoutGLCanvas::OnPaint )
    //EVT_ERASE_BACKGROUND( LayoutGLCanvas::OnErase )
    EVT_MOTION  ( LayoutGLCanvas::OnMouseMotion )
    EVT_LEFT_DOWN   ( LayoutGLCanvas::OnLeftDown )
    EVT_LEFT_UP ( LayoutGLCanvas::OnLeftUp )
    EVT_RIGHT_DOWN  ( LayoutGLCanvas::OnRightDown )
    EVT_RIGHT_UP    ( LayoutGLCanvas::OnRightUp )
    EVT_MIDDLE_DOWN  ( LayoutGLCanvas::OnMiddleDown )
    EVT_MOUSEWHEEL ( LayoutGLCanvas::OnMouseWheel )
    EVT_KEY_DOWN    ( LayoutGLCanvas::OnKeyDown )
    EVT_KEY_UP  ( LayoutGLCanvas::OnKeyUp )
	EVT_CHAR  ( LayoutGLCanvas::OnChar )
	EVT_ENTER_WINDOW (LayoutGLCanvas::OnEnterWindow)
	EVT_LEAVE_WINDOW (LayoutGLCanvas::OnLeaveWindow)
    EVT_SCROLLWIN( LayoutGLCanvas::OnScrollWin )
END_EVENT_TABLE()

//-----------------------------------------------------------------------
LayoutGLCanvas::LayoutGLCanvas( EditorView* v, wxWindow* parent, const wxPoint& pos, const wxSize& size ) :
    wxGLCanvas( parent, static_cast<const wxGLContext *>( 0 ), -1, pos, size , wxSUNKEN_BORDER ),  // | wxVSCROLL | wxHSCROLL ),
    m_view( v ),
    m_GUISystem( 0 ),
    m_GUILayoutRenderer( 0 ),
    m_layoutImage( 0 ),
    m_realWidth( 0 ),
    m_realHeight( 0 ),
	m_layoutZoomFactor(1),
    m_scrollPosX( 0 ),
    m_scrollDocX( 0 ),
    m_scrollPageX( 0 ),
    m_scrollPosY( 0 ),
    m_scrollDocY( 0 ),
	m_font(0),
	m_scheme(0),
    m_scrollPageY( 0 )
{
    // Init one-time-only stuff
    SetCurrent();
    glShadeModel( GL_SMOOTH );
    glEnable( GL_DEPTH_TEST );

    // we will use a cross-hair cursor
    SetCursor( wxCURSOR_BLANK);

    // Get the CEGUI system (already initialize from EditorLayout)

	m_GUISystem = CEGUI::System::getSingletonPtr();
	m_GUILayoutRenderer = (OpenGLRenderer*) m_GUISystem->getRenderer();

}

//-----------------------------------------------------------------------
LayoutGLCanvas::~LayoutGLCanvas()
{
    // Close the test Layout

}

//-----------------------------------------------------------------------
void LayoutGLCanvas::loadLayout( const wxString& layoutName,const wxString& schemeName,const wxString& fontName )
{
		SetCurrent();

		/*if (m_layoutImage)
		{
			CEGUI::WindowManager::getSingletonPtr()->destroyWindow(m_layoutImage);
			m_layoutImage = NULL;
		}*/

        
		try
		{
			if (m_scheme)
				CEGUI::SchemeManager::getSingleton().unloadScheme(m_scheme->getName());
			m_scheme = CEGUI::SchemeManager::getSingleton().loadScheme(CEGUIHelper::ToCEGUIString(schemeName));
		}
		catch ( Exception& guiException )
		{
			wxLogError ( wxT( "%s" ), CEGUIHelper::ToWXString( guiException.getMessage() ).c_str() );
		}

		// warn user if this layout has no relation with the working document
		ImagesetManager& isMgr = ImagesetManager::getSingleton();
		const String imagesetName ( CEGUIHelper::ToCEGUIString( m_view->getPropsPanel()->getImagesetName() ) );
		if ( !isMgr.isImagesetPresent( imagesetName) )
		{
			wxMessageBox(wxT("The current ImageSet is not not using the layout you're about to open!\n"),
				wxT("Warning"), wxICON_INFORMATION | wxOK);
		}

        // Set default mouse cursor: Must be defined as MouseArrow or MouseCursor in the ImageSet

		CEGUI::ImagesetManager* imgsetmgr = CEGUI::ImagesetManager::getSingletonPtr();
		CEGUI::ImagesetManager::ImagesetIterator iter = imgsetmgr->getIterator();
		while( !iter.isAtEnd() )
		{
			if ((*iter)->isImageDefined("MouseArrow"))
			{
				const CEGUI::Image& img = (*iter)->getImage("MouseArrow");
				CEGUI::System::getSingletonPtr()->setDefaultMouseCursor(&img);
				break;
			}
			if ((*iter)->isImageDefined("MouseCursor"))
			{
				const CEGUI::Image& img = (*iter)->getImage("MouseCursor");
				CEGUI::System::getSingletonPtr()->setDefaultMouseCursor(&img);
				break;
			}
			iter++;
		} 
		m_mouseCursor = CEGUI::System::getSingletonPtr()->getDefaultMouseCursor();

		// Load the font

		if (!fontName.IsEmpty())
		{
			try
			{
				if (m_font)
					FontManager::getSingleton().destroyFont(m_font);
				m_font = FontManager::getSingleton().createFont(CEGUIHelper::ToCEGUIString(fontName));
				CEGUI::String scaled = m_font->getProperty("AutoScaled");
				if(scaled=="True")
				{
					defaultAutoScaled = true;

					wxMessageBox(wxT("Your autoscaled font attribute is set to true. This feature is causing major memery leaks...\nIt is strongly adviced to set it to false before using the editor."),
						wxT("Dangerous font attribute"), wxICON_WARNING | wxOK);
				}
				else
					defaultAutoScaled = false;
			}
			catch ( Exception& guiException )
			{
				wxLogError ( wxT( "%s" ), CEGUIHelper::ToWXString( guiException.getMessage() ).c_str() );
			}
			m_GUISystem->setDefaultFont(m_font);
		}

		try
		{
			m_layoutImage = CEGUI::WindowManager::getSingletonPtr()->loadWindowLayout(CEGUIHelper::ToCEGUIString(layoutName));
		}
		catch ( Exception& guiException )
		{
			wxLogError ( wxT( "%s" ), CEGUIHelper::ToWXString( guiException.getMessage() ).c_str() );
			return;
		}

        m_layoutImage->setMaxSize( UVector2( UDim( 0.0f, 20480.0f ), UDim( 0.0f, 20480.0f ) ) );
        m_layoutImage->setPosition( UVector2( UDim( 0.0f, 0.0f ), UDim( 0.0f, 0.0f ) ) );

        // Find the root
		CEGUI::Window* root = CEGUI::WindowManager::getSingletonPtr()->getWindow("__editor_root_window__");
		// Add a check later!
		root->addChildWindow(m_layoutImage);

		// save layout data in ini file

		CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_LAST_SCHEME),schemeName);
		CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_LAST_FONT),fontName);
		CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_LAST_LAYOUT),layoutName);



		Render();

}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnResize( wxSizeEvent& event )
{
	SetCurrent();
    updateScrollbars();
}

//-----------------------------------------------------------------------
bool LayoutGLCanvas::resetCEGUI()
{
	SetCurrent();

    // reset image size to 0
    m_realWidth = 640;
    m_realHeight = 480;
	if (m_layoutImage)
		m_layoutImage->setSize(
			UVector2( UDim( 0, m_realWidth ), UDim( 0, m_realHeight ) ) );


	if (CEGUI::WindowManager::getSingletonPtr()->isWindowPresent("__editor_root_window__"))
	{
		CEGUI::Window* root = CEGUI::WindowManager::getSingletonPtr()->getWindow("__editor_root_window__");
		unsigned int childCount = root->getChildCount();
		for (unsigned int i=0; i<childCount; i++)
		{
			wxString windowName = CEGUIHelper::ToWXString(root->getChildAtIdx(i)->getName());
			if (windowName.Cmp(wxT("__editor_static_image__")))
			{
				root->getChildAtIdx(i)->destroy();
			}
		}
	}

	if (m_layoutImage)
	{
		CEGUI::WindowManager::getSingletonPtr()->destroyWindow(m_layoutImage);
		m_layoutImage = NULL;
	}

	if (m_scheme)
	{
		CEGUI::SchemeManager::getSingleton().unloadScheme(m_scheme->getName());
		m_scheme = NULL;
	}

	if (m_font)
	{
		FontManager::getSingleton().destroyFont(m_font);
		m_font = NULL;
	}

    return true;
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnMouseMotion( wxMouseEvent& event )
{
    this->SetFocus();

	m_GUISystem->setDefaultMouseCursor(m_mouseCursor);


    // tell CEGUI about the event
    System::getSingleton().injectMousePosition( event.GetX(), event.GetY() );

    // Check this, because we might get this event during document closing
    if ( isDocumentValid() &&
        ( event.GetX() != m_lastMouseX || event.GetY() != m_lastMouseY ) )
    {
        m_lastMouseX = event.GetX();
        m_lastMouseY = event.GetY();
    }

    Render();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnLeftDown( wxMouseEvent& event )
{
    // Check this, because we might get this event during document closing
    if ( isDocumentValid() )
    {
        // tell CEGUI about the event
        System::getSingleton().injectMouseButtonDown( CEGUI::LeftButton );
        Render();
    }

    event.Skip();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnLeftUp( wxMouseEvent& event )
{
    // Check this, because we might get this event during document closing
    if ( isDocumentValid() )
    {
        // tell CEGUI about the event
        System::getSingleton().injectMouseButtonUp( CEGUI::LeftButton );
        Render();
    }

    event.Skip();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnRightDown( wxMouseEvent& event )
{
    // Check this, because we might get this event during document closing
    if ( isDocumentValid() )
    {
        // tell CEGUI about the event
        System::getSingleton().injectMouseButtonDown( CEGUI::RightButton );
        Render();
    }

    event.Skip();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnRightUp( wxMouseEvent& event )
{
    // Check this, because we might get this event during document closing
    if ( isDocumentValid() )
    {
        // tell CEGUI about the event
        System::getSingleton().injectMouseButtonUp( CEGUI::RightButton );
        Render();
    }

    event.Skip();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnMiddleDown( wxMouseEvent& event )
{
    // initialise point where dragging started
    event.Skip();
}


//-----------------------------------------------------------------------
bool LayoutGLCanvas::Reset()
{
    return resetCEGUI();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::Render()
{

	SetCurrent();
	// scriptkid :hide entire layout; only show this window

	if (CEGUI::WindowManager::getSingletonPtr()->isWindowPresent("__editor_root_window__"))
	{
		CEGUI::Window* root = CEGUI::WindowManager::getSingletonPtr()->getWindow("__editor_root_window__");
		unsigned int childCount = root->getChildCount();
		for (unsigned int i=0; i<childCount; i++)
		{
			root->getChildAtIdx(i)->setVisible(false);
		}
	}

	if (m_layoutImage)
	{
		m_layoutImage->setVisible(true);
		int clientWidth, clientHeight;
		GetClientSize( &clientWidth, &clientHeight );
		
		CEGUI::Size sz( static_cast<float>( clientWidth ),static_cast<float>( clientHeight ) );
		m_GUILayoutRenderer->setDisplaySize(sz);

	}

	// set layout font to Autoscaled=true before rendering if needed
	if (m_font)
	{
		if(defaultAutoScaled==true)
			m_font->setProperty("AutoScaled","true");
	}

    // CEGUI rendering of the Left Layout only
	renderCEGUILayout();
}

//-----------------------------------------------------------------------
bool LayoutGLCanvas::isDocumentValid() const
{
    return (m_view && m_view->GetDocument());
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnPaint( wxPaintEvent& event )
{
    // We need to do this to avoid a flood of paint events
    wxPaintDC paintDC( this );
    Render();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::renderCEGUILayout()
{

    // clear colour buffer
    glClear( GL_COLOR_BUFFER_BIT );

	if ( CEGUI::System::getSingletonPtr() )
	{
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		CEGUI::System::getSingleton().renderGUI();
	}

    glFlush();
    SwapBuffers();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnErase( wxEraseEvent& event )
{
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnKeyDown( wxKeyEvent& event )
{
	switch (event.GetKeyCode())
	{
		case WXK_BACK:
		case WXK_DELETE:
			m_GUISystem->injectKeyDown(CEGUI::Key::Backspace);
		break;

		case WXK_RETURN:
			m_GUISystem->injectKeyDown(CEGUI::Key::Return);
		break;

		default:
			m_GUISystem->injectKeyDown(event.GetKeyCode());
		break;
	}

	event.Skip();	// needed for the char event to happen
	Render();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnKeyUp( wxKeyEvent& event )
{
	m_GUISystem->injectKeyUp(event.GetKeyCode());
	Render();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnChar( wxKeyEvent& event )
{
	m_GUISystem->injectChar( event.GetUnicodeKey());

	Render();
}


//-----------------------------------------------------------------------
void LayoutGLCanvas::OnLeaveWindow( wxMouseEvent& event)
{
    if ( m_layoutImage )
    {
        // tell CEGUI about the event
        m_GUISystem->setDefaultMouseCursor(NULL);
        Render();
    }

    event.Skip();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnEnterWindow( wxMouseEvent& event)
{
	if ( m_layoutImage )
    {
        // tell CEGUI about the event
        m_GUISystem->setDefaultMouseCursor(m_mouseCursor);
        Render();
    }

    event.Skip();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::OnMouseWheel( wxMouseEvent& event )
{
    // Check this, because we might get this event during document closing
    if ( isDocumentValid() )
    {
        // FIXME: Add a real zoom Layout support later
		/*int MouseWheelDirection = event.GetWheelRotation();


		if ( MouseWheelDirection > 0 )
			m_layoutZoomFactor += 0.03;
		else if ( MouseWheelDirection < 0 )
			m_layoutZoomFactor -= 0.03;

		// extrem down limit verif
		if ( m_layoutZoomFactor < 0.25 )
			m_layoutZoomFactor = 0.25;

		// extrem up limit verif
		if ( m_layoutZoomFactor > 10.0 )
			m_layoutZoomFactor = 10.0;

		float width = m_layoutZoomFactor * m_realWidth;
		float height = m_layoutZoomFactor * m_realHeight;

		// mute events to stop us from generating a lot of unrequired noise
		CEGUI::GlobalEventSet::getSingleton().setMutedState( true );
		// update the size of the main image
		m_layoutImage->setSize(
			UVector2( cegui_absdim( width ), cegui_absdim( height ) ) );
		// allow events to fire again.
		CEGUI::GlobalEventSet::getSingleton().setMutedState( false );

        Render();*/
    }
}



//-----------------------------------------------------------------------
void LayoutGLCanvas::OnScrollWin( wxScrollWinEvent& event )
{
	int orientation = event.GetOrientation();
    int adjustment, page, range, position;

    if ( orientation == wxVERTICAL )
    {
        page = m_scrollPageY;
        range = m_scrollDocY;
        position = m_scrollPosY;
    }
    else
    {
        page = m_scrollPageX;
        range = m_scrollDocX;
        position = m_scrollPosX;
    }

    wxEventType evtype = event.GetEventType();

    if ( evtype == wxEVT_SCROLLWIN_TOP )
        adjustment = -position;
    else if ( evtype == wxEVT_SCROLLWIN_BOTTOM )
        adjustment = range - page - position;
    else if ( evtype == wxEVT_SCROLLWIN_LINEDOWN )
        adjustment = 1;
    else if ( evtype == wxEVT_SCROLLWIN_LINEUP )
        adjustment = -1;
    else if ( evtype == wxEVT_SCROLLWIN_PAGEDOWN )
        adjustment = page;
    else if ( evtype == wxEVT_SCROLLWIN_PAGEUP )
        adjustment = -page;
    else if ( evtype == wxEVT_SCROLLWIN_THUMBTRACK )
        adjustment = event.GetPosition() - position;
    else if ( evtype == wxEVT_SCROLLWIN_THUMBRELEASE )
        adjustment = event.GetPosition() - position;
    else
        adjustment = 0;

    position += adjustment;

    if ( orientation == wxVERTICAL )
        m_scrollPosY = position;
    else
        m_scrollPosX = position;

    updateScrollbars();
    updateLayoutScrollPosition();
    event.Skip();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::setBackgroundColour( const wxColour& colour )
{
    SetCurrent();
	
	glClearColor(
        colour.Red() / 255.0f,
        colour.Green() / 255.0f,
        colour.Blue() / 255.0f,
        1.0f
    );
    //Render();
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::updateScrollbars()
{
    
	// update internal tracking of this information.
    GetClientSize( &m_scrollPageX, &m_scrollPageY );
    m_scrollDocX = static_cast<int>( m_realWidth);
    m_scrollDocY = static_cast<int>( m_realHeight);

    // update scrolly bars
    SetScrollbar( wxVERTICAL, m_scrollPosY, m_scrollPageY, m_scrollDocY );
    SetScrollbar( wxHORIZONTAL, m_scrollPosX, m_scrollPageX, m_scrollDocX );

    // finally, update our local copies (saves doing our own constraints)
    m_scrollPosX = GetScrollPos( wxHORIZONTAL );
    m_scrollPosY = GetScrollPos( wxVERTICAL );
}

//-----------------------------------------------------------------------
void LayoutGLCanvas::updateLayoutScrollPosition()
{
    m_layoutImage->setPosition( UVector2(
        UDim( 0, 0 - m_scrollPosX ), UDim( 0, 0 - m_scrollPosY ) ) );

    Render();
}
