///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

// CEGUI Includes
#include <CEGUI.h>
#include <CEGUIDefaultResourceProvider.h>


// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#include "wx/image.h"
#include "wx/filename.h"

#include "LayoutPanel.h"
#include "EditorGLCanvas.h"
#include "LayoutGLCanvas.h"
#include "EditorView.h" 
// Utility class for convertion between CEGUI- and wxWidgets types
#include "CEGUIHelper.h"
//#include "Options.h"
#include "CEImagesetConfig.h"

// Toolbar icons
#include "bitmaps/new.xpm"
#include "bitmaps/open.xpm"
#include "bitmaps/save.xpm"
#include "bitmaps/cut.xpm"
#include "bitmaps/copy.xpm"
#include "bitmaps/paste.xpm"
#include "bitmaps/delete.xpm"
#include "bitmaps/help.xpm"
#include "bitmaps/shape_square_add.xpm"
#include "bitmaps/shape_square_delete.xpm"

// ID codes for the widgets we use on the panel
const int ID_TEXT = 10000;
const int ID_IMAGESETNAME_TEXTCTRL = 10001;
const int ID_IMAGE_FILENAME_TEXTCTRL = 10002;
const int ID_IMAGE_FILENAME_BROWSE_BUTTON = 10003;
const int ID_NATIVE_RESOLUTION_COMBO = 10004;
const int ID_AUTOSCALE_CHECKBOX = 10005;
const int ID_GRID = 10006;
const int ID_SCROLLED = 10007;
const int ID_GRID_ADD_ROW = 10008;
const int ID_GRID_DEL_ROW = 10009;

const int ID_TEXT_LAYOUT					= 10010;
const int ID_LAYOUT_FILENAME_TEXTCTRL		= 10011;
const int ID_LAYOUT_FILENAME_BROWSE_BUTTON  = 10012;

const int ID_TEXT_SCHEME					= 10013;
const int ID_SCHEME_FILENAME_TEXTCTRL		= 10014;
const int ID_SCHEME_FILENAME_BROWSE_BUTTON  = 10015;

const int ID_TEXT_FONT						= 10016;
const int ID_FONT_FILENAME_TEXTCTRL			= 10017;
const int ID_FONT_FILENAME_BROWSE_BUTTON	= 10018;

const int ID_REFRESH_LAYOUT_BUTTON			= 10019;


// wxWidgets event bindings table
BEGIN_EVENT_TABLE( LayoutPanel, wxPanel )
	EVT_BUTTON( ID_LAYOUT_FILENAME_BROWSE_BUTTON, LayoutPanel::onBrowseLayoutFile )
	EVT_BUTTON( ID_SCHEME_FILENAME_BROWSE_BUTTON, LayoutPanel::onBrowseSchemeFile )
	EVT_BUTTON( ID_FONT_FILENAME_BROWSE_BUTTON,	LayoutPanel::onBrowseFontFile )
	EVT_BUTTON( ID_REFRESH_LAYOUT_BUTTON, LayoutPanel::onRefreshLayout )
END_EVENT_TABLE()

//-----------------------------------------------------------------------
LayoutPanel::LayoutPanel( wxWindow* parent, const wxPoint& pos, const wxSize& size ) :
    wxPanel( parent, wxID_ANY, pos, size ),
    m_view( 0 )
{}

//-----------------------------------------------------------------------
void LayoutPanel::setView(EditorView* aView)
{
    m_view = aView;
}

//-----------------------------------------------------------------------
void LayoutPanel::createContent()
{

    // to manage the layout of the props panel
    wxBoxSizer* sizerProps = new wxBoxSizer( wxVERTICAL );
    SetSizer( sizerProps );

    // sets layout properties section
    wxStaticText* lblLayoutSet = new wxStaticText( this, ID_TEXT, wxT( "Test Layout" ), wxDefaultPosition, wxSize( 250, -1 ), wxALIGN_CENTRE );
    lblLayoutSet->SetBackgroundColour( *wxGREEN );
    sizerProps->Add( lblLayoutSet, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL, 0 );

    // ----------------------------------------------------------------------------
    // Font file selector
	wxStaticText* lblFontFile = new wxStaticText( this, ID_TEXT_FONT, wxT( "Font file :" ), wxDefaultPosition, wxDefaultSize, 0 );
    sizerProps->Add( lblFontFile, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT | wxTOP, 5 );
    wxBoxSizer* sizerFontFile = new wxBoxSizer( wxHORIZONTAL );

    m_fontFilenameTextCtrl = new wxTextCtrl( this, ID_FONT_FILENAME_TEXTCTRL, wxT( "" ), wxDefaultPosition, wxSize( 175, -1 ), wxTE_PROCESS_ENTER );
    sizerFontFile->Add( m_fontFilenameTextCtrl, 1, wxGROW | wxALIGN_CENTER_HORIZONTAL | wxALL, 0 );

    m_fontFilenameBrowseButton = new wxButton( this, ID_FONT_FILENAME_BROWSE_BUTTON, wxT( "..." ), wxDefaultPosition, wxSize( 20, -1 ), 0 );
    sizerFontFile->Add( m_fontFilenameBrowseButton, 0, wxALIGN_BOTTOM | wxALIGN_CENTER_HORIZONTAL | wxALL, 0 );

    // add the scheme file selector to the props panel
    sizerProps->Add( sizerFontFile, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5 );

    // ----------------------------------------------------------------------------
    // Scheme file selector
	wxStaticText* lblSchemeFile = new wxStaticText( this, ID_TEXT_SCHEME, wxT( "Sheme file :" ), wxDefaultPosition, wxDefaultSize, 0 );
    sizerProps->Add( lblSchemeFile, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT | wxTOP, 5 );
    wxBoxSizer* sizerSchemeFile = new wxBoxSizer( wxHORIZONTAL );

    m_schemeFilenameTextCtrl = new wxTextCtrl( this, ID_SCHEME_FILENAME_TEXTCTRL, wxT( "" ), wxDefaultPosition, wxSize( 175, -1 ), wxTE_PROCESS_ENTER );
    sizerSchemeFile->Add( m_schemeFilenameTextCtrl, 1, wxGROW | wxALIGN_CENTER_HORIZONTAL | wxALL, 0 );

    m_schemeFilenameBrowseButton = new wxButton( this, ID_SCHEME_FILENAME_BROWSE_BUTTON, wxT( "..." ), wxDefaultPosition, wxSize( 20, -1 ), 0 );
    sizerSchemeFile->Add( m_schemeFilenameBrowseButton, 0, wxALIGN_BOTTOM | wxALIGN_CENTER_HORIZONTAL | wxALL, 0 );

    // add the shceme file selector to the props panel
    sizerProps->Add( sizerSchemeFile, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5 );


    // ----------------------------------------------------------------------------
    // layout file selector
	wxStaticText* lblLayoutFile = new wxStaticText( this, ID_TEXT_LAYOUT, wxT( "Layout file :" ), wxDefaultPosition, wxDefaultSize, 0 );
    sizerProps->Add( lblLayoutFile, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT | wxTOP, 5 );
    wxBoxSizer* sizerLayoutFile = new wxBoxSizer( wxHORIZONTAL );

    m_layoutFilenameTextCtrl = new wxTextCtrl( this, ID_LAYOUT_FILENAME_TEXTCTRL, wxT( "" ), wxDefaultPosition, wxSize( 175, -1 ), wxTE_PROCESS_ENTER );
    sizerLayoutFile->Add( m_layoutFilenameTextCtrl, 1, wxGROW | wxALIGN_CENTER_HORIZONTAL | wxALL, 0 );

    m_layoutFilenameBrowseButton = new wxButton( this, ID_LAYOUT_FILENAME_BROWSE_BUTTON, wxT( "..." ), wxDefaultPosition, wxSize( 20, -1 ), 0 );
    sizerLayoutFile->Add( m_layoutFilenameBrowseButton, 0, wxALIGN_BOTTOM | wxALIGN_CENTER_HORIZONTAL | wxALL, 0 );

    // add the layout file selector to the props panel
    sizerProps->Add( sizerLayoutFile, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5 );

    // ----------------------------------------------------------------------------
    // Refresh layout Button

	m_refreshLayoutButton = new wxButton( this, ID_REFRESH_LAYOUT_BUTTON, wxT( "Load - Refresh" ), wxDefaultPosition, wxDefaultSize, 0 );
	sizerProps->Add( m_refreshLayoutButton, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 5 );

    // set initial state
    reset();

	// Init fields with last loaded files
	setLayoutFilename( CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_LAST_LAYOUT)) );
	setSchemeFilename( CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_LAST_SCHEME)) );
	setFontFilename( CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_LAST_FONT)) );
}

//-----------------------------------------------------------------------
void LayoutPanel::reset()
{
}

//-----------------------------------------------------------------------
wxString LayoutPanel::getLayoutFilename() const
{
    return m_layoutFilenameTextCtrl->GetValue();
}

//-----------------------------------------------------------------------
void LayoutPanel::setLayoutFilename( const wxString& name )
{
    m_layoutFilenameTextCtrl->SetValue( name );
}

//-----------------------------------------------------------------------
wxString LayoutPanel::getSchemeFilename() const
{
    return m_schemeFilenameTextCtrl->GetValue();
}

//-----------------------------------------------------------------------
void LayoutPanel::setSchemeFilename( const wxString& name )
{
    m_schemeFilenameTextCtrl->SetValue( name );
}

//-----------------------------------------------------------------------
wxString LayoutPanel::getFontFilename() const
{
    return m_fontFilenameTextCtrl->GetValue();
}

//-----------------------------------------------------------------------
void LayoutPanel::setFontFilename( const wxString& name )
{
    m_fontFilenameTextCtrl->SetValue( name );
}

//-----------------------------------------------------------------------
bool LayoutPanel::isDocumentValid() const
{
    return ( m_view && m_view->GetDocument() );
}

//-----------------------------------------------------------------------
EditorDocument* LayoutPanel::getDocument() const
{
    return m_view ?
        static_cast<EditorDocument*>( m_view->GetDocument() ) :
        0;
}

//-----------------------------------------------------------------------
void LayoutPanel::onBrowseLayoutFile( wxCommandEvent& WXUNUSED( event ) )
{

	wxString fullPath = wxT("");
    CEGUI::DefaultResourceProvider* rp = static_cast<CEGUI::DefaultResourceProvider*>(CEGUI::System::getSingleton().getResourceProvider());
	fullPath = CEGUIHelper::ToWXString(rp->getResourceGroupDirectory( CEGUI::WindowManager::getDefaultResourceGroup() ) );
	wxFileName currFilespec(fullPath);



    // Show file selector.
    wxString filename = wxFileSelector( wxT( "Open Layout" ),
                                        currFilespec.GetPath(),
                                        currFilespec.GetFullName(),
                                        wxT( "Layout" ),
                                        wxT( "Layout files (*.layout)|*.layout|all files (*.*)|*.*" ) );

    // if something was selected
    if ( !filename.empty() )
    {
        filename = getLayoutResourceGroupRelativePath( filename );
        // update the filename in the dialog
        setLayoutFilename( filename );
    }
}

//-----------------------------------------------------------------------
void LayoutPanel::onBrowseSchemeFile( wxCommandEvent& WXUNUSED( event ) )
{

	wxString fullPath = wxT("");
    CEGUI::DefaultResourceProvider* rp = static_cast<CEGUI::DefaultResourceProvider*>(CEGUI::System::getSingleton().getResourceProvider());
	fullPath = CEGUIHelper::ToWXString(rp->getResourceGroupDirectory( CEGUI::Scheme::getDefaultResourceGroup() ) );
	wxFileName currFilespec(fullPath);

    // Show file selector.
    wxString filename = wxFileSelector( wxT( "Open Scheme" ),
                                        currFilespec.GetPath(),
                                        currFilespec.GetFullName(),
                                        wxT( "Scheme" ),
                                        wxT( "Scheme files (*.scheme)|*.scheme|all files (*.*)|*.*" ) );

    // if something was selected
    if ( !filename.empty() )
    {
		filename = getSchemeResourceGroupRelativePath( filename );
        // update the filename in the dialog
        setSchemeFilename( filename );
    }
}

//-----------------------------------------------------------------------
void LayoutPanel::onBrowseFontFile( wxCommandEvent& WXUNUSED( event ) )
{
	wxString fullPath = wxT("");
    CEGUI::DefaultResourceProvider* rp = static_cast<CEGUI::DefaultResourceProvider*>(CEGUI::System::getSingleton().getResourceProvider());
	fullPath = CEGUIHelper::ToWXString(rp->getResourceGroupDirectory( CEGUI::Font::getDefaultResourceGroup() ) );
	wxFileName currFilespec(fullPath);

    // Show file selector.
    wxString filename = wxFileSelector( wxT( "Open Font" ),
                                        currFilespec.GetPath(),
                                        currFilespec.GetFullName(),
                                        wxT( "Font" ),
                                        wxT( "Font files (*.font)|*.font|all files (*.*)|*.*" ) );

    // if something was selected
    if ( !filename.empty() )
    {
        filename = getFontResourceGroupRelativePath( filename );
        // update the filename in the dialog
        setFontFilename( filename );
    }
}

//-----------------------------------------------------------------------
void LayoutPanel::onRefreshLayout( wxCommandEvent& WXUNUSED( event ) )
{
    if ( isDocumentValid() )
		getDocument()->setLayoutFilename( getLayoutFilename(), getSchemeFilename(), getFontFilename() );
}

//-----------------------------------------------------------------------
wxString LayoutPanel::getDefaultResourceGroupRelativePath( const wxString& filename ) const
{
    wxString resourcePath( wxT( "./" ) );
    wxFileName filespec( filename );

    CEGUI::DefaultResourceProvider* rp =
        static_cast<CEGUI::DefaultResourceProvider*>(
            CEGUI::System::getSingleton().getResourceProvider() );

    CEGUI::String defResGrp = rp->getDefaultResourceGroup();

    if ( !defResGrp.empty() )
        resourcePath = CEGUIHelper::ToWXString(
            rp->getResourceGroupDirectory( defResGrp ) );

    if ( filename.empty() )
    {
        return resourcePath;
    }
    else
    {
        filespec.MakeRelativeTo( resourcePath );
        return filespec.GetFullPath();
    }
}

//-----------------------------------------------------------------------
wxString LayoutPanel::getLayoutResourceGroupRelativePath( const wxString& filename ) const
{
    wxString resourcePath( wxT( "./" ) );
    wxFileName filespec( filename );

    CEGUI::DefaultResourceProvider* rp =
        static_cast<CEGUI::DefaultResourceProvider*>(
            CEGUI::System::getSingleton().getResourceProvider() );
    
	CEGUI::String defResGrp = CEGUI::WindowManager::getDefaultResourceGroup();

    if ( !defResGrp.empty() )
        resourcePath = CEGUIHelper::ToWXString(
            rp->getResourceGroupDirectory( defResGrp ) );

    if ( filename.empty() )
    {
        return resourcePath;
    }
    else
    {
        filespec.MakeRelativeTo( resourcePath );
        return filespec.GetFullPath();
    }
}

//-----------------------------------------------------------------------
wxString LayoutPanel::getSchemeResourceGroupRelativePath( const wxString& filename ) const
{
    wxString resourcePath( wxT( "./" ) );
    wxFileName filespec( filename );

    CEGUI::DefaultResourceProvider* rp =
        static_cast<CEGUI::DefaultResourceProvider*>(
            CEGUI::System::getSingleton().getResourceProvider() );
    
	CEGUI::String defResGrp = CEGUI::Scheme::getDefaultResourceGroup();

    if ( !defResGrp.empty() )
        resourcePath = CEGUIHelper::ToWXString(
            rp->getResourceGroupDirectory( defResGrp ) );

    if ( filename.empty() )
    {
        return resourcePath;
    }
    else
    {
        filespec.MakeRelativeTo( resourcePath );
        return filespec.GetFullPath();
    }
}

//-----------------------------------------------------------------------
wxString LayoutPanel::getFontResourceGroupRelativePath( const wxString& filename ) const
{
    wxString resourcePath( wxT( "./" ) );
    wxFileName filespec( filename );

    CEGUI::DefaultResourceProvider* rp =
        static_cast<CEGUI::DefaultResourceProvider*>(
            CEGUI::System::getSingleton().getResourceProvider() );
    
	CEGUI::String defResGrp = CEGUI::Font::getDefaultResourceGroup();

    if ( !defResGrp.empty() )
        resourcePath = CEGUIHelper::ToWXString(
            rp->getResourceGroupDirectory( defResGrp ) );

    if ( filename.empty() )
    {
        return resourcePath;
    }
    else
    {
        filespec.MakeRelativeTo( resourcePath );
        return filespec.GetFullPath();
    }
}

