///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////
#ifndef _LAYOUT_PANEL_H_
#define _LAYOUT_PANEL_H_

// Standard wxWidget includes
#include "wx/wx.h" 

#include "EditorDocument.h"


class EditorView;

class LayoutPanel: public wxPanel
{
public:
    //! Constructor
    LayoutPanel(wxWindow* parent, const wxPoint& pos, const wxSize& size);

    //! Sets the view which we are a part of.
    void setView (EditorView* aView);

    //! Create the widgets that populate the LayoutPanel.
    void createContent();

    //! Resets the LayoutPanel to it's initial state.
    void reset();

    //
    // The following are all GUI handler members; they react to changes
    // in the panel GUI and update the editor document accordingly.
    //
    // Note that these members should not directly update anything
    // relating to the view, that will happen via the document's
    // response to being updated.
    //


		void onRefreshLayout(wxCommandEvent& WXUNUSED(event));
		void onBrowseLayoutFile(wxCommandEvent& WXUNUSED(event));
		void onBrowseSchemeFile(wxCommandEvent& WXUNUSED(event));
		void onBrowseFontFile(wxCommandEvent& WXUNUSED(event));


	wxString getLayoutFilename() const;
	void setLayoutFilename(const wxString& name);

	wxString getSchemeFilename() const;
	void setSchemeFilename(const wxString& name);

	wxString getFontFilename() const;
	void setFontFilename(const wxString& name);

private:
    //! returns whether we have access to a valid EditorDocument object.
    bool isDocumentValid() const;

    //! returns the current EditorDocument object.
    EditorDocument* getDocument() const;


    //! Return the given path 'filename' relative to the wanted resource group directory
    wxString getDefaultResourceGroupRelativePath( const wxString& filename ) const;
	wxString getLayoutResourceGroupRelativePath( const wxString& filename ) const;
	wxString getSchemeResourceGroupRelativePath( const wxString& filename ) const;
	wxString getFontResourceGroupRelativePath( const wxString& filename ) const;

    //
    // Data fields
    //
    //! The view to which we are attached.
    EditorView* m_view;
    //! Text control that holds the file name of the test layout.
    wxTextCtrl* m_layoutFilenameTextCtrl;
    wxButton* m_layoutFilenameBrowseButton;

    wxTextCtrl* m_schemeFilenameTextCtrl;
    wxButton* m_schemeFilenameBrowseButton;

    wxTextCtrl* m_fontFilenameTextCtrl;
    wxButton* m_fontFilenameBrowseButton;

		wxButton* m_refreshLayoutButton;


    DECLARE_EVENT_TABLE()
};

#endif // _PROPERTIES_PANEL_H_
